// -------------------------------------------------------------
// ArcBall
// -------------------------------------------------------------

class ArcBall
{
  float contextWidth, contextHeight;

  vector3 circlePointStart, circlePointDrag;
  vector3 spherePointStart, spherePointDrag;

  Quaternion qStart, qRotation, qFinal;
  float[] mRotation;

  ArcBall(float w, float h)
  {
    contextWidth    = w;
    contextHeight   = h;
  
    circlePointStart = new vector3();
    spherePointStart = new vector3();

    circlePointDrag = new vector3();
    spherePointDrag = new vector3();
  
    qRotation = new Quaternion();
    qStart = new Quaternion();
    qFinal = new Quaternion();
  
    mRotation = new float[9];
    for (int i=0;i<9;i++) mRotation[i] = 0.0f;
    mRotation[0] = 1.0f;mRotation[4] = 1.0f;mRotation[8] = 1.0f;
  }
  
  vector3 getPointSphere(vector3 P)
  {
	float r = P.x*P.x + P.y*P.y;
        vector3 result = new vector3();
    
        result.x = P.x;
        result.y = P.y;
        result.z = P.z;    
        
	if(r>1.0)
	{	float sr = sqrt(r);
		result.x /= sr;
		result.y /= sr;	
		result.z = 0.0;
	}
	else
		result.z = sqrt( 1.0-r);  

    

    return result;
  
  }

  Quaternion getQuat(vector3 pStartPoint, vector3 pEndPoint)
  {
      Quaternion  result       = new Quaternion();
      vector3     crossResult  = new vector3();
      float       dotResult;
        
 
	crossResult = cross(pStartPoint,pEndPoint);
        dotResult   = pStartPoint.x*pEndPoint.x + pStartPoint.y*pEndPoint.y + pStartPoint.z*pEndPoint.z;
        
	result.set(dotResult, crossResult);  
      
      return result;
  }

  void start(Quaternion qStart, float x, float y)
  {
    this.qStart.set(qStart);
    
    circlePointStart.x = (x - 0.5*contextWidth) / (0.5*contextWidth);
    circlePointStart.y = (y - 0.5*contextHeight) / (0.5*contextHeight);
    circlePointDrag.x   = circlePointStart.x;
    circlePointDrag.y   = circlePointStart.y;

   spherePointStart      = getPointSphere(circlePointStart);
   spherePointDrag.set(spherePointStart);
  }

  Quaternion update(float x, float y)
  {
    circlePointDrag.x = (x - 0.5*contextWidth) / (0.5*contextWidth);
    circlePointDrag.y = (y - 0.5*contextHeight) / (0.5*contextHeight);
    spherePointDrag   = getPointSphere(circlePointDrag);
  
    qRotation = getQuat(spherePointStart, spherePointDrag);
    qFinal  = multiply(qRotation, qStart);
    qFinal.normalize();
    mRotation = qFinal.toMatrix3();    
    return qFinal;
  }
  
  float[] getRotationMatrix()
  {
    return mRotation;
  }
     
}
